(* ::Package:: *)

AppendTo[$Path,"/home/guanxin/mmapkg/external/LiteRed"];
AppendTo[$Path,"/home/guanxin/mmapkg/external/finiteflow-mathtools-master/packages"];
AppendTo[$Path,"/home/guanxin/mmapkg/external/finiteflow"];
AppendTo[$LibraryPath,"/usr/local/lib"];
If[$FrontEnd===Null,$InputFileName,NotebookFileName[]]//DirectoryName//SetDirectory;
<<LiteIBP.m;
<<topology.wl;


exints = Import["ibps/exints_def.m"];
all = Join[LIBPGetAllInts[FileNames["ibps/ints_dbox_*.mx"]], First/@exints];
eqsfiles = FileNames["ibps/sids_*.json"];
vars = {s, t, eps};


(*integrals up to rank 3 and 1 dotted propagators*)
(*we choose this integral set to determine masters and we guess it is sufficient to compute differential equations*)
target=Select[all,LIBPIntS[#]<=3&&(LIBPIntR[#]-LIBPIntT[#]==1)&];


(* write system.json with all the info on the system *)
LIBPWriteSystemJSON[eqsfiles, all, target, vars, "FileName"->"system.json"];


FFNewGraph[graph,in,vars];

(* create a linear solver form system.json *)
FFAlgJSONSparseSolver[graph,ibps,{in},"system.json"];

(* don't include constant term in the (numerical) solution *)
FFSolverOnlyHomogeneous[graph,ibps];
FFGraphOutput[graph,ibps];

(* Learning phase: get the master integrals *)
ibplearn = FFSparseSolverLearn[graph,all];
{nonmis, mis} = {"DepVars", "IndepVars"}/.ibplearn;
Print["number of integrals" -> Length/@{nonmis,mis}];
mastersexpr = mis/.exints;


mastersexpr


invariants = {s, t};
LIBPComputeDerivatives[dbox];


(* Derivative of all the masters with respect to x *)
computederivative[x_]:=LIBPEliminateZeroSectors[Collect[LIBPDeriv[#,x]&/@mastersexpr,_j,Together]];
derivatives = computederivative/@invariants;


new = LIBPIntegralsIn[derivatives];
new = Join[new,mis];


(* The new list of needed integrals must be a subset of the previous one *)
If[!SubsetQ[Join[nonmis,mis],new],Print["error: current ibp system is not sufficient to reduce all derivatives."];Abort[]];


(*reset the list of target integrals*)
FFSolverResetNeededVars[graph,ibps,all,new];
newibplearn = FFSparseSolverLearn[graph,all];
If[mis =!= ("IndepVars"/.newibplearn), Print["error: derivatives are reduced to a different set of master integrals."];Abort[]];
{nonmis, mis} = {"DepVars", "IndepVars"} /. newibplearn;


(*trimming*)
FFSparseSolverMarkAndSweepEqs[graph,ibps];
FFSparseSolverDeleteUnneededEqs[graph,ibps];
Print["number of independent eqs."->FFSolverNIndepEqs[graph,ibps]];


ibpintegrals = Join[nonmis,mis];


(*derivatives may depend on masters explictly. so we append reduction rules of master integrals to master integrals*)
FFAlgRatNumEval[graph,misred,Join@@IdentityMatrix[Length[mis]]];
FFAlgChain[graph,ibpsfull,{ibps,misred}];


(*encode the coefficients of the unreduced derivatives of the masters w.r.t. the full ibp integrals*)
FFAlgRatFunEval[graph, unreduced, {in},vars,Join@@(FFLinearCoefficients[#,ibpintegrals]&/@(Join@@derivatives))];


FFGraphDraw[graph]


(*matrix multiplication algorithm, to finally get the differential equations*)
FFAlgMatMul[graph,de,{unreduced,ibpsfull},Length[invariants]*Length[mis], Length[ibpintegrals], Length[mis]];


(*Since differential equations are sparse, we append a NonZero algorithm which only takes the non-zero elements*)
FFAlgNonZeroes[graph,denz,{de}];
FFGraphOutput[graph,denz];
nzlearn = FFNonZeroesLearn[graph];


FFGraphDraw[graph]


(*recontruction*)
rec0 = FFReconstructFunction[graph,vars,"MaxPrimes"->200,"MaxDegree"->1000];
rec = Normal[FFNonZeroesSol[rec0,nzlearn]];


rec//Length


(*get differential equations*)
deall = Factor@ArrayReshape[rec,{Length[invariants],Length[mis],Length[mis]}];
Do[
  de[invariants[[i]]] = deall[[i]];
,{i,Length@invariants}];


(* Check some expected properties of the result *)
de[s].de[t]-de[t].de[s]//Together;
D[de[s],t]-D[de[t],s]//Together;

%+%%//Simplify


FFDeleteGraph[graph];
Quit[];
