(* ::Package:: *)

(*dimensional regularization*)
SetDim[4-2eps];

(*loop momentum*)
Internal = {l1, l2};

(*external momentum*)
External = {k1, k2, k3, k4};

MomentumConservation = {mm[k3] -> -mm[k1] - mm[k2] - mm[k4]};

(*scalar product to kinematic invariants*)
Replacements = {mmp[k1, k2] -> s/2, mmp[k1, k4] -> t/2, mmp[k2, k4] -> (-s - t)/2, mmp2[k1] -> 0, mmp2[k2] -> 0, mmp2[k4] -> 0};

(*definition of kinematic invariants*)
Invariants = {s -> 2*mmp[k1, k2], t -> 2*mmp[k1, k4]};

(*Type of momentum: Vector, Type of kinematic invariants(or mass) : Number*)
Declare[#,Number]&/@{s, t};


(*support linear propagators *)
Propagators = {mmp2[l1], -2*mmp[l1, k1] + mmp2[k1] + mmp2[l1], 2*mmp[k1, k2] - 2*mmp[l1, k1] - 2*mmp[l1, k2] + mmp2[k1] + mmp2[k2] + mmp2[l1], 2*mmp[k1, k2] + 2*mmp[l2, k1] + 2*mmp[l2, k2] + mmp2[k1] + mmp2[k2] + mmp2[l2], -2*mmp[l2, k4] + mmp2[k4] + mmp2[l2], mmp2[l2], 2*mmp[l1, l2] + mmp2[l1] + mmp2[l2], 2*mmp[l1, k4] + mmp2[k4] + mmp2[l1], 2*mmp[l2, k1] + mmp2[k1] + mmp2[l2]}/. MomentumConservation/.Replacements//Expand;

(*dbox: family name*)
LIBPFamilyLite[dbox, Propagators, Internal, External, MomentumConservation, Replacements, Invariants];
NewBasis[dbox,LIBPToLiteRed[Propagators],LIBPLoopMomenta[dbox],GenerateIBP->True];

(*CutDs: cut propagators, useful when apply Reverse Unitarity*)
(*analyze zerosectors, symmetry relations ...*)
AnalyzeSectors[dbox,{_, _, _, _, _, _, _, 0, 0},CutDs->{0, 0, 0, 0, 0, 0, 0, 0, 0}];

LIBPFindSymmetries[dbox,EMs->LIBPEMs[dbox]];
